{%- extends "!layout.html" %}

{%- block breadcrumbs %}
{{ super() }}
<!--REGION_DISCLAIMER_DO_NOT_REMOVE-->
{%- endblock %}


{%- block footer %}
{{ super() }}
<script type="text/javascript" src="https://media.amazonwebservices.com/js/sitecatalyst/s_code.min.js"></script>
<script type="text/javascript">
s.prop66 = 'Adapter to enable developing Django apps with Aurora DSQL';
s.eVar66 = 'D=c66';
s.prop65 = 'API Reference';
s.eVar65 = 'D=c65';
var s_code = s.t();
if (s_code) document.write(s_code);
</script>
{%- endblock %}