<div class="sidebar-block">
    <div class="sidebar-wrapper">
      <h2>{{ _('Table Of Contents') }}</h2>
    </div>
    <div class="sidebar-toc">
      {% set toctree = toctree(includehidden=True) %}
      {% if toctree %}
        {{ toctree }}
      {% else %}
        {{ toc }}
      {% endif %}
    </div>
  </div>